#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <math.h>
#include <string.h>
#include <time.h>

#define INIT_DIT_CAPACITY 5


typedef struct {
int day, month, year;
} date_t;


typedef struct {
char name[40];
char surname[40];
date_t birthday;
int phone_number;
} person_t;

typedef struct {
int size;
int capacity;
person_t *content;
} directory_t;


// Prints a person record with the format "Name Surname, day-month-year, phone number"
// Parameter : p - the person
void person_print(person_t p){

    printf("%s %s, %d/%d/%d,%d \n",p.name,p.surname,p.birthday.day,p.birthday.month,p.birthday.year,p.phone_number);


}
// Creates a new directory with size 0 and capacity INIT_DIT_CAPACITY

void dir_init(directory_t *dir){

    dir->size=0;
    dir->capacity= INIT_DIT_CAPACITY;

    if (dir->content == NULL) {
        dir->content = malloc(sizeof(person_t) * dir->capacity);
    } else {
        dir->content = realloc(dir->content, sizeof(person_t) * dir->capacity);
    }  

}

directory_t dir_create(){

    directory_t newDirectory;
    newDirectory.content = NULL;

    dir_init(&newDirectory);
    
    return newDirectory;

}
// Free the memory of the directory, set size and capacity to 0 and content to NULL
// Parameter : dir - the directory
void dir_free(directory_t *dir){

    dir->capacity=0;
    dir->size=0;
    free(dir->content);
}
// Prints the content of a directory each line with the format "[n] person"
// with n being the position in the directory
// Parameters : dir - the directory

void dir_print(directory_t *dir,int n){


    if(n==-1){
            for (int i = 0; i < dir->size; i++)
        {
            person_t t = dir->content[i];
            printf("[%d] ",i);
            person_print(t);
        }
    }else {
        
        person_t t = dir->content[n];
            printf("[%d] ",n);
            person_print(t);
    }
}


// Erase a directory by setting its size to 0, but letting its capacity untouched
// Parameter : dir - the directory
void dir_erase(directory_t *dir){

    dir->size=0;

}

// Add a person to the directory if its capacity is not reached and increase its size
// If the capacity is reached, attempt to increase the capacity before adding the person
// Parameters : dir - the directory, p - the person
// Returns : 0 if success, 1 otherwise
int dir_add_entry(directory_t *dir, person_t p){

    if (dir->size >= dir->capacity) {
        person_t *new_p = realloc(dir->content, (dir->size + 1) * sizeof(person_t));
        if (new_p == NULL) {
            return 1;
        }
        dir->content = new_p;
        dir->capacity += 1;
    }

    dir->content[dir->size] = p;
    dir->size++;

    return 0;

}

// Parameter : dir - the direcory
// Returns : 0 if success, 1 otherwise
int dir_fill(directory_t *dir){

dir_init(dir);

if(dir_add_entry(dir, (person_t){"Jean", "Paul", (date_t){18, 12, 1967}, 12346890})) return 1;
if(dir_add_entry(dir, (person_t){"Arthur", "Rimbaud", (date_t){1, 1, 2002}, 73645})) return 1;
if(dir_add_entry(dir, (person_t){"Paul", "Albuquerque", (date_t){23, 6, 1902}, 28736474})) return 1;
if(dir_add_entry(dir, (person_t){"Christophe", "Charpilloz", (date_t){11, 4, 2008}, 76253})) return 1;
if(dir_add_entry(dir, (person_t){"Emily", "Dickinson", (date_t){19, 8, 1950}, 636363})) return 1;
if(dir_add_entry(dir, (person_t){"Ada", "Lovelace", (date_t){12, 12, 2022}, 12345})) return 1;
if(dir_add_entry(dir, (person_t){"Qwertz", "Uiop", (date_t){1, 1, 562}, 67879})) return 1;
//printf("current dir size : %d \n", dir->size);
return 0;

}


// Delete the entry at position n of the directory if n is smaller than
// the size of the directory, does nothing otherwise
// Parameters : dir - the direcory, n - the position
void dir_delete_entry(directory_t *dir, int n){
    if(n < dir->size){

        for (int  i = 0; i< dir->size-1; i++)
        {
            dir->content[i]=dir->content[i+1];
        }

        dir->size--;

    } 

    printf("Contact removed from Annuary.\n");

}
// Searches for an entry with a given surname
// Parameters : dir - the directory, surname - the surname to search
// Returns : the index of a matching entry, a negative number if no match is found
int dir_search_surname(directory_t* dir, char* surname){

    for (int i = 0; i < dir->size; i++)
    {
        person_t p = dir->content[i];
        if (strcmp(surname, dir->content[i].surname) == 0) {
            return i;
        }
    }
    return -1;

}

int main(){

    
    directory_t directory = dir_create();
    
    dir_fill(&directory);


    printf("Available commands : add, del, show, search, quit,\n");
    char command[255]=" ";
    while(strcmp(command,"quit")!=0){
        printf("> ");
        scanf("%s", command);

    
        if(strcmp(command,"add")==0){
            person_t new_p;

            printf("Enter the name: ");
            scanf("%s",new_p.name);
            printf("Enter the surname: ");
            scanf("%s",new_p.surname);
            printf("Enter the day of birth: ");
            scanf("%i",&new_p.birthday.day);
            printf("Enter the month of birth: ");
            scanf("%d",&new_p.birthday.month);
            printf("Enter the year of birth: ");
            scanf("%d",&new_p.birthday.year);
            printf("Enter the phone number: ");
            scanf("%d",&new_p.phone_number);
            

            dir_add_entry(&directory,new_p);

        }else if(strcmp(command,"del")==0){
            int index;

            printf("Enter index to remove: ");
            scanf("%d",&index);
            dir_delete_entry(&directory,index);

            printf("Contact removed from Annuary.\n");

        }else if(strcmp(command,"show")==0){

            char options[40]="";
            printf("enter the name to show or all if you want to show entire directory: \n" );
            scanf("%s",options);
            int index=-1;

            if (strcmp(options,"all")==0){
                
                dir_print(&directory,index);
            }else {
                
                index= dir_search_surname(&directory,options);
                dir_print(&directory,index);
                
                };
            

        }else if(strcmp(command,"search")==0){
            int index=-1;
            char Surname[255];
            printf(" Enter Surname: ");
            scanf("%s",Surname);

            dir_search_surname(&directory,Surname);

            if(index>=0){

                printf("The Surname index is : %d\n",index);
            }else{

                printf("The Surname index doesn't exist");
            }
        }
        else if(strcmp(command,"quit")==0){
            dir_erase(&directory);

            printf("Directory erased");
        }
    }
    dir_free(&directory);

}