#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <time.h>

// Enumeration representing parts of the maze
typedef enum { WALL, PATH, START, END, USED_PATH } maze_part_t;

// Structure representing the maze
typedef struct {
    maze_part_t** data;
    int n;
} maze_t;

// Function declarations
void maze_print(maze_t maze);
maze_t maze_alloc(int n);
void maze_free(maze_t maze);
void maze_create(maze_t maze);
bool dig_maze(maze_t maze, int i, int j);
bool solve_maze(maze_t maze, int i, int j);

int main() {
    // Seed for randomization
    srand(time(NULL));

    // Example usage
    int maze_size = 10;  // You can change the size as needed
    maze_t my_maze = maze_alloc(maze_size);

    // Create and print the maze
    maze_create(my_maze);
    dig_maze(my_maze,1,1);
    maze_print(my_maze);

    // Solve the maze (bonus)
    if (solve_maze(my_maze, 0, my_maze.n - 1)) {
        printf("\nMaze solved!\n");
        maze_print(my_maze);
    } else {
        printf("\nNo solution found.\n");
    }

    // Free allocated memory
    maze_free(my_maze);

    return 0;
}

// Function to print the maze
void maze_print(maze_t maze) {
    for (int i = 0; i < maze.n; ++i) {
        for (int j = 0; j < maze.n; ++j) {
            switch (maze.data[i][j]) {
                case WALL:
                    printf("█ ");
                    break;
                case PATH:
                    printf("  ");
                    break;
                case START:
                    printf("S ");
                    break;
                case END:
                    printf("E ");
                    break;
                case USED_PATH:
                    printf(". ");
                    break;
            }
        }
        printf("\n");
    }
}

// Function to allocate memory for a maze and initialize it with WALLs
maze_t maze_alloc(int n) {
    maze_t maze;
    maze.n = n;

    // Allocate memory for the maze
    maze.data = (maze_part_t**)malloc(n * sizeof(maze_part_t*));
    for (int i = 0; i < n; ++i) {
        maze.data[i] = (maze_part_t*)malloc(n * sizeof(maze_part_t));
    }

    // Initialize all cells to WALL
    for (int i = 0; i < n; ++i) {
        for (int j = 0; j < n; ++j) {
            maze.data[i][j] = WALL;
        }
    }

    return maze;
}

// Function to free memory of a maze
void maze_free(maze_t maze) {
    for (int i = 0; i < maze.n; ++i) {
        free(maze.data[i]);
    }
    free(maze.data);
}

// Function to create the maze following specified criteria
void maze_create(maze_t maze) {
    // Set the start and end points
    maze.data[maze.n - 1][maze.n / 2] = START;
    maze.data[0][maze.n / 2] = END;

    // Dig tunnels
    dig_maze(maze, maze.n - 1, maze.n / 2);
}

// Function to recursively dig tunnels in the maze
bool dig_maze(maze_t maze, int i, int j) {
    // Check if the position is out of bounds or has been visited
    if (i < 0 || i >= maze.n || j < 0 || j >= maze.n || maze.data[i][j] != WALL) {
        return false;
    }

    // If the current position is the end, return true
    if (maze.data[i][j] == END) {
        return true;
    }

    // Change the current position to PATH
    maze.data[i][j] = PATH;

    // Define directions (north, south, east, west)
    int directions[4][2] = {{-1, 0}, {1, 0}, {0, 1}, {0, -1}};

    // Shuffle the directions randomly
    for (int k = 0; k < 4; ++k) {
        int rand_dir = rand() % 4;
        int new_i = i + directions[rand_dir][0];
        int new_j = j + directions[rand_dir][1];

        // Recursively dig in the chosen direction
        if (dig_maze(maze, new_i, new_j)) {
            return true;
        }
    }

    // If all directions have been tried and no path found, backtrack
    maze.data[i][j] = USED_PATH;
    return false;
}

// Function to solve the maze (bonus)
bool solve_maze(maze_t maze, int i, int j) {
    // TODO: Implement maze solving logic using backtracking
}
